﻿using CD14Roma_EF.Models;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations.Schema;
using System.Data.Entity;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CD14Roma_EF.Data
{
    public class ApplicationDbContext : DbContext
    {
        public DbSet<SessionModel> Sessions { get; set; }
        public DbSet<EnrollmentModel> Enrollments { get; set; }
        public DbSet<AttendeeModel> Attendees { get; set; }
        public DbSet<ArgumentModel> Arguments { get; set; }
        public DbSet<SpeakerModel> Speakers { get; set; }

        public ApplicationDbContext()
            : base("DefaultConnection")
        {
            Configuration.ProxyCreationEnabled = false;
            Configuration.LazyLoadingEnabled = false;
        }

        protected override void OnModelCreating(DbModelBuilder modelBuilder)
        {
            var session = modelBuilder.Entity<SessionModel>();

            session.ToTable("Sessions");

            session.HasKey(e => e.Id);
            session.Property(e => e.Id).HasDatabaseGeneratedOption(DatabaseGeneratedOption.Identity);
            session.Property(e => e.Id).IsRequired();

            session.Property(e => e.Title).IsRequired();
            session.Property(e => e.Title).HasMaxLength(255);

            session.Property(e => e.StartAt).IsRequired();
            session.Property(e => e.StartAt).HasColumnType("smalldatetime");

            session.Property(e => e.Duration).IsRequired();

            session.Property(e => e.Description).IsOptional();
            session.Property(e => e.Description).HasMaxLength(1000);

            session.HasRequired(e => e.Argument).WithMany().Map(e => e.MapKey("ArgumentID"));
            session.HasRequired(e => e.Speaker).WithMany().Map(t => t.MapKey("SpeakerID")).WillCascadeOnDelete(false);

            var enrollment = modelBuilder.Entity<EnrollmentModel>();

            enrollment.ToTable("Enrollments");

            enrollment.HasKey(e => e.Id);
            enrollment.Property(e => e.Id).HasDatabaseGeneratedOption(DatabaseGeneratedOption.Identity);
            enrollment.Property(e => e.Id).IsRequired();

            enrollment.HasOptional(e => e.Attendee).WithMany(e => e.Enrollments).Map(e => e.MapKey("AttendeeID")).WillCascadeOnDelete(false);
            enrollment.HasOptional(e => e.Session).WithMany(e => e.Enrollments).Map(e => e.MapKey("SessionID")).WillCascadeOnDelete(false);

            var attendee = modelBuilder.Entity<AttendeeModel>();

            attendee.ToTable("Attendees");

            attendee.HasKey(e => e.Id);
            attendee.Property(e => e.Id).HasDatabaseGeneratedOption(DatabaseGeneratedOption.Identity);
            attendee.Property(e => e.Id).IsRequired();

            attendee.Property(e => e.Email).IsRequired();
            attendee.Property(e => e.Email).HasMaxLength(255);
            attendee.Property(e => e.Email).IsUnicode(false);

            attendee.Property(e => e.UserName).IsRequired();
            attendee.Property(e => e.UserName).HasMaxLength(50);
            attendee.Property(e => e.UserName).IsUnicode(false);

            attendee.Property(e => e.Password).IsRequired();
            attendee.Property(e => e.Password).HasMaxLength(255);

            attendee.Property(e => e.FirstName).IsRequired();
            attendee.Property(e => e.FirstName).HasMaxLength(50);

            attendee.Property(e => e.LastName).IsRequired();
            attendee.Property(e => e.LastName).HasMaxLength(50);

            var argument = modelBuilder.Entity<ArgumentModel>();

            argument.ToTable("Arguments");

            argument.HasKey(e => e.Id);
            argument.Property(e => e.Id).HasDatabaseGeneratedOption(DatabaseGeneratedOption.Identity);
            argument.Property(e => e.Id).IsRequired();

            argument.Property(e => e.Description).IsRequired();
            argument.Property(e => e.Description).HasMaxLength(255);

            var speaker = modelBuilder.Entity<SpeakerModel>();

            speaker.ToTable("Speakers");

            speaker.HasKey(e => e.Id);
            speaker.Property(e => e.Id).HasDatabaseGeneratedOption(DatabaseGeneratedOption.Identity);
            speaker.Property(e => e.Id).IsRequired();

            speaker.Property(e => e.FirstName).IsRequired();
            speaker.Property(e => e.FirstName).HasMaxLength(50);

            speaker.Property(e => e.LastName).IsRequired();
            speaker.Property(e => e.LastName).HasMaxLength(50);

            base.OnModelCreating(modelBuilder);
        }

        public Task<bool> AddRefreshToken(RefreshToken token)
        {
            throw new NotImplementedException();
        }

        public Task<RefreshToken> FindRefreshToken(string hashedTokenId)
        {
            throw new NotImplementedException();
        }

        public Task<RefreshToken> RemoveRefreshToken(string hashedTokenId)
        {
            throw new NotImplementedException();
        }
    }
}
