﻿using CD14Roma_EF.Data;
using CD14Roma_EF.Models;
using CD14Roma_WebAPI.ViewModels;
using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Data.Entity.Infrastructure;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;
using System.Web.Http;
using System.Web.Http.Routing;

namespace CD14Roma_WebAPI.Controllers
{
    public class SpeakersController : ApiController
    {
        private ApplicationDbContext db = new ApplicationDbContext();

        #region Get
        public IEnumerable<object> Get()
        {
            return db.Speakers
                .ToList()
                .Select(s => new
                {
                    FirstName = s.FirstName,
                    LastName = s.LastName
                });
        }

        public IHttpActionResult Get(int id)
        {
            var speaker = db.Speakers
                .Find(id);
            if (speaker == null)
                return NotFound();
            return Ok(new {
                    FirstName = speaker.FirstName,
                    LastName = speaker.LastName
                }
            );
        }

        //public IHttpActionResult Get(int speakerId)
        //{
        //    var speaker = db.Speakers
        //        .Find(speakerId);
        //    if (speaker == null)
        //        return NotFound();
        //    return Ok(new SpeakerViewModel(speaker));
        //}
        #endregion

        #region Post
        public IHttpActionResult Post([FromBody] SpeakerViewModel speaker)
        {
            if (!ModelState.IsValid)
            {
                return BadRequest(ModelState);
            }

            var model = speaker.ToModel();

            db.Speakers.Add(model);
            db.SaveChanges();

            return CreatedAtRoute("DefaultApi", new { id = model.Id }, speaker);
        }
        #endregion

        #region Delete
        public async Task<IHttpActionResult> Delete(int speakerId)
        {
            //try
            //{
                var speaker = await db.Speakers
                    .FindAsync(speakerId);
                //if (speaker == null)
                //{
                //    return NotFound();
                //}

                db.Speakers.Remove(speaker);
                await db.SaveChangesAsync();

                return Ok(speaker);
            //}
            //catch (Exception ex)
            //{
            //    while (ex.InnerException != null)
            //        ex = ex.InnerException;
            //    return InternalServerError(ex);
            //}
        }
        #endregion

        #region Put
        public async Task<IHttpActionResult> Put(int speakerId, SpeakerViewModel speaker)
        {
            if (!ModelState.IsValid)
            {
                return BadRequest(ModelState);
            }

            var model = speaker.ToModel(speakerId);

            db.Entry(model).State = EntityState.Modified;

            try
            {
                await db.SaveChangesAsync();
            }
            catch (DbUpdateConcurrencyException)
            {
                if (!SpeakerExists(speakerId))
                {
                    return NotFound();
                }
                else
                {
                    throw;
                }
            }

            return StatusCode(HttpStatusCode.NoContent);
        }
        #endregion

        #region GetSessions
        [Route("api/Speakers/{speakerId}/Sessions")]
        public IHttpActionResult GetSessions(int speakerId)
        {
            var speaker = db.Speakers
                .Where(s => s.Id == speakerId);

            if (speaker == null)
                return NotFound();

            var sessions = db.Sessions
                .Where(s => s.Speaker.Id == speakerId);

            return Ok(sessions);
        }
        #endregion

        #region _private
        private bool SpeakerExists(int id)
        {
            return db.Speakers
                .Count(e => e.Id == id) > 0;
        }
        #endregion
    }
}
